/*
 * Canvas subclass for drawing polygons.
 *
 * Copyright 1997, Steve Robbins
 */

/**
 * These are I/O classes for polygons.  The user can draw, with the
 * mouse, a polygonal chain (PolylineCanvas) or closed polygon
 * (PolygonCanvas) on the canvas.  The polygon is stored (in
 * class Polygon) as a chain of vertices.  For a PolygonCanvas, the
 * first and last vertices will coincide.
 *
 * The canvas can have a "controller" object, which:
 * - is notified when the user has finished drawing the polygon
 * - is called at the beginning of the "paint" method to draw on the canvas
 *
 */

import java.awt.*;

public class PolylineCanvas extends Canvas
{
    /**
     * The input Polygon may be read by the public, but should not be
     * written.  Java doesn't seem to have a "const" modifier, so I
     * don't know how to enforce this.
     */
    public Polygon inputPolygon = new Polygon();

    protected int lastX, lastY, curX, curY;
    protected boolean drawable;
    protected CanvasController controller;
    
    public PolylineCanvas()
    {
	super();
	reset();
	setBackground(Color.white);
    }

    public void setFinishNotify(CanvasController c)
    {
	controller = c;
    }
    
    protected void finish()
    {
	lastX = curX = -1;
	drawable = false;
	if (controller != null)
	    controller.finishNotify(this);
	repaint();
    }

    public void reset()
    {
	lastX = curX = -1;
	drawable = true;
	inputPolygon = new Polygon();
	repaint();
    }
	
    public boolean mouseDown(Event event, int x, int y)
    {
	if (!drawable)  return false;
	
	if (event.modifiers == 0) { // left button
	    inputPolygon.addPoint(x, y);
	    lastX = curX = -1;
	    repaint();
	} else { // middle or right button --> finish polygon
	    finish();
	}
	return false;
    }

    public boolean mouseMove(Event event, int x, int y)
    {
	if (!drawable)  return false;

	lastX = curX;
	lastY = curY;
	curX = x;
	curY = y;
	repaint();
	return false;
    }
    
    public void paint(Graphics g)
    {
	g.setPaintMode();
	g.setColor(getForeground());

	if (inputPolygon.npoints > 0)
	{
	    g.drawPolygon(inputPolygon);

	    // Find coordinates of last polygon vertex
	    int px = inputPolygon.xpoints[inputPolygon.npoints-1];
	    int py = inputPolygon.ypoints[inputPolygon.npoints-1];

	    /*
	      if (lastX != -1) { // erase last drag-line
	      g.setXORMode(getBackground());
	      g.drawLine(px, py, lastX, lastY);
	      g.setPaintMode();
	      }
	      */
	    
	    if (curX != -1) { // draw current drag-line
		g.drawLine(px, py, curX, curY);
	    }
	}
	
	if (controller != null)
	    controller.paint(this, g);

    }
}

